-- regwrite.vhd
--
-- Version 0.0 2001/10/30
-- adopted from pllmode_write.vhd
-- Address 428: JPBURSTREG

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;


ENTITY regwrite IS
    PORT(
        rst		: IN	STD_LOGIC;
        CLK		: IN	STD_LOGIC;
        phase		: IN	STD_LOGIC;
        aux1		: IN	STD_LOGIC;
        we		: IN	STD_LOGIC;
        din		: IN	std_logic_vector(15 downto 0);
        bstcount   : out   std_logic_vector(5 downto 0);
        spccount   : out   std_logic_vector(5 downto 0);
        version     : out   std_logic
	);
END regwrite;

ARCHITECTURE a OF regwrite IS

    signal header_state : std_logic;
    signal dcount : std_logic_vector(10 downto 0);
    signal endcount : std_logic_vector(10 downto 0);
    signal dinreg : std_logic_vector(15 downto 0);
    signal address : std_logic_vector(15 downto 0);
    signal wereg  : std_logic;
    signal reg :std_logic_vector(15 downto 0) := "0000000000000000";
BEGIN

-- decode input data (dinreg)
   bstcount <= reg(5 downto 0);
   spccount <= reg(11 downto 6);
   version <= reg(12);
-- input data registers 
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (phase = '1' and aux1 = '0' ) then
                dinreg <= din;
                wereg <=we;
            else
                dinreg <= dinreg;
                wereg <= wereg;
            end if;
        end if;
    end process;

-- header state
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (phase = '1' and aux1 = '0') then
                if (rst = '0') then
                     header_state <= '0';
                elsif (wereg = '0') then
                    if (header_state = '0' and dcount = 0) then
                        header_state <= '1';
                    else
                        header_state <= '0';
                    end if;
                end if;
            else
                header_state <=header_state;
            end if;
        end if;
    end process;
    
-- data counter
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (phase = '1' and aux1 = '0') then
                if (rst = '0') then
                    dcount <= "00000000000";
                elsif (wereg = '0') then
                    if (header_state = '1') then
                        dcount <= dinreg(10 downto 0);
                    elsif (dcount /= 0) then
                        dcount <= dcount - 1;
                    end if;
                end if;
            else
                dcount <= dcount;
            end if;
        end if;
    end process;

-- address counter 
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (phase = '1' and aux1 = '0') then
                if (wereg = '0') then
                    if( header_state = '0' and dcount = 0 ) then
                        address<= dinreg;
                    elsif( header_state = '0' ) then
                        address <= address + 1;
                    end if;
                end if;
            else
                address <= address;
            end if;
        end if;
    end process;

-- load data
    process(clk)
    begin
        if(clk'event and clk='1') then
-- use 0x428 for JPBURSTMODE
            if (phase = '1' and aux1 = '0') then
                if (wereg = '0' and address = 16#0428# and dcount /=0 ) then
                    reg <=dinreg;
                end if;
            else
                    reg <= reg;
            end if;
        end if;
    end process;
END a;











