-- fof
-- snooping the IP link and get necessary informations
--
-- Version 0.0 1998/9/15 J. Makino

-- MAKUTIL ADD_PARITY 36

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;


ENTITY ip_snoop IS
    PORT(
        rst		: IN	STD_LOGIC;
        CLK		: IN	STD_LOGIC;
        we		: IN	STD_LOGIC;
        din		: IN	std_logic_vector(15 downto 0);
        n               : out   std_logic_vector(7 downto 0);
        calc_start      : out   std_logic
	);
END ip_snoop;

ARCHITECTURE a OF ip_snoop IS

    signal header_state : std_logic;
    signal dcount : std_logic_vector(10 downto 0);
    signal endcount : std_logic_vector(10 downto 0);
    signal dinreg : std_logic_vector(15 downto 0);
    signal address : std_logic_vector(15 downto 0);
    signal wereg  : std_logic;

BEGIN

-- input data registers 
    process(clk)
    begin
        if(clk'event and clk='1') then
            dinreg <= din;
            wereg <=we;
        end if;
    end process;

-- header state
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (rst = '0') then
                header_state <= '0';
            elsif (wereg = '0') then
                if (header_state = '0' and dcount = 0) then
                    header_state <= '1';
                else
                    header_state <= '0';
                end if;
            end if;
        end if;
    end process;
    
-- data counter
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (rst = '0') then
                dcount <= "00000000000";
            elsif (wereg = '0') then
                if (header_state = '1') then
                    dcount <= dinreg(10 downto 0);
                elsif (dcount /= 0) then
                    dcount <= dcount - 1;
                end if;
            end if;
        end if;
    end process;

-- address counter 
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (wereg = '0') then
                if( header_state = '0' and dcount = 0 ) then
                    address<= dinreg;
                elsif( header_state = '0' ) then
                    address <= address + 1;
                end if;
            end if;
        end if;
    end process;

-- load N for the address of N
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (wereg = '0' and address = 16#1012# and dcount /=0 ) then
                n<= dinreg(7 downto 0);
            end if;
        end if;
    end process;

-- assert (set high) calc_start when 
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (wereg = '0' and address = 16#1402# and dcount /= 0) then
                calc_start <= '1';
            else
                calc_start <= '0';
            end if;
        end if;
    end process;
END a;











