-- pllmode_write.vhd
--
-- Version 0.0 2001/5/20
-- adopted from fofpga_write2.vhd
-- Address 426: B0PLLMODE (6:0)
-- Address 427: B1PLLMODE (4:0)

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;


ENTITY pllmode_write IS
    PORT(
        rst		: IN	STD_LOGIC;
        CLK		: IN	STD_LOGIC;
        we		: IN	STD_LOGIC;
        din		: IN	std_logic_vector(15 downto 0);
            dindum      : in    std_logic;
        b0pllmode   : out   std_logic_vector(6 downto 0);
        b1pllmode   : out   std_logic_vector(4 downto 0)
	);
END pllmode_write;

ARCHITECTURE a OF pllmode_write IS

    signal header_state : std_logic;
    signal dcount : std_logic_vector(10 downto 0);
    signal endcount : std_logic_vector(10 downto 0);
    signal dinreg : std_logic_vector(15 downto 0);
    signal address : std_logic_vector(15 downto 0);
    signal wereg  : std_logic;
    signal b0pllreg :std_logic_vector(15 downto 0) := "0000000001010101";
    signal b1pllreg :std_logic_vector(15 downto 0) := "0000000001010101";
BEGIN

-- decode input data (dinreg)
   b0pllmode(0) <= '0' when b0pllreg(1 downto 0) = "00"
                else 'Z' when b0pllreg(1 downto 0) = "01"
                else '1';
   b0pllmode(1) <= '0' when b0pllreg(3 downto 2) = "00"
                else 'Z' when b0pllreg(3 downto 2) = "01"
                else '1';
   b0pllmode(2) <= '0' when b0pllreg(5 downto 4) = "00"
                else 'Z' when b0pllreg(5 downto 4) = "01"
                else '1';
   b0pllmode(3) <= '0' when b0pllreg(7 downto 6) = "00"
                else 'Z' when b0pllreg(7 downto 6) = "01"
                else '1';
   b0pllmode(6 downto 4) <= "ZZZ" when dindum = '0' else "111"; 

   b1pllmode(0) <= '0' when b1pllreg(1 downto 0) = "00"
                else 'Z' when b1pllreg(1 downto 0) = "01"
                else '1';
   b1pllmode(1) <= '0' when b1pllreg(3 downto 2) = "00"
                else 'Z' when b1pllreg(3 downto 2) = "01"
                else '1';
   b1pllmode(2) <= '0' when b1pllreg(5 downto 4) = "00"
                else 'Z' when b1pllreg(5 downto 4) = "01"
                else '1';
   b1pllmode(3) <= '0' when b1pllreg(7 downto 6) = "00"
                else 'Z' when b1pllreg(7 downto 6) = "01"
                else '1';
   b1pllmode(4) <= 'Z' when dindum  = '0' else '1' ;

-- input data registers 
    process(clk)
    begin
        if(clk'event and clk='1') then
            dinreg <= din;
            wereg <=we;
        end if;
    end process;

-- header state
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (rst = '0') then
                header_state <= '0';
            elsif (wereg = '0') then
                if (header_state = '0' and dcount = 0) then
                    header_state <= '1';
                else
                    header_state <= '0';
                end if;
            end if;
        end if;
    end process;
    
-- data counter
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (rst = '0') then
                dcount <= "00000000000";
            elsif (wereg = '0') then
                if (header_state = '1') then
                    dcount <= dinreg(10 downto 0);
                elsif (dcount /= 0) then
                    dcount <= dcount - 1;
                end if;
            end if;
        end if;
    end process;

-- address counter 
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (wereg = '0') then
                if( header_state = '0' and dcount = 0 ) then
                    address<= dinreg;
                elsif( header_state = '0' ) then
                    address <= address + 1;
                end if;
            end if;
        end if;
    end process;

-- load N for the address of N
    process(clk)
    begin
        if(clk'event and clk='1') then
-- use 0x426 and 0x427 for b0pll and b1pll
--                                
            if (wereg = '0' and address = 16#0426# and dcount /=0 ) then
                b0pllreg <=dinreg;
            end if;
            if (wereg = '0' and address = 16#0427# and dcount /=0 ) then
                b1pllreg <=dinreg;
            end if;
        end if;
    end process;
END a;











