-- G6HIB main controller FPGA
-- Rev          : 4.0.0
-- cont2.vhd
-- specialized for CB/PB version 2
-- always sending out same data from both ip/jp port
-- Version 2001/4/22
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity cont2 is
  port(

--basic-----------------------------------
      BCLK          :in       std_logic;                      --from clock
      BRSTn         :out      std_logic;                      --to all parts
--9080------------------------------------
      LHOLD         :in       std_logic;                      --from 9080
      ADSn          :in       std_logic;                      --from 9080
      BLASTn        :in       std_logic;                      --from 9080
      LWR           :in       std_logic;                      --from 9080
      RESETn        :in       std_logic;                      --from 9080
      LHOLDA        :out      std_logic;                      --to 9080
      READYin       :out      std_logic;                      --to 9080
      LAD           :inout    std_logic_vector(31 downto 0);  --to & from 9080
      PCHKn         :in       std_logic;                      --from 9080
      LRESETn       :out      std_logic;                      --to 9080
      DP            :in       std_logic_vector(3 downto 0);
      nDEN          :in       std_logic;                      --to & from 9080

--DPRAM-----------------------------------
      RAM_DATA      :in       std_logic_vector(31 downto 0);  --from DPRAM
      RAM_PAR       :in       std_logic_vector( 3 downto 0);  --from DPRAM
      COUNT_OUT     :out      std_logic_vector(14 downto 0);  --to DRRAM write
      RAM_ADR       :out      std_logic_vector(14 downto 0);  --to DPRAM read
      WYn           :out      std_logic;                      --to DPRAM
--FIFO------------------------------------
      EFn1          :in       std_logic;                      --from FIFO#1
      EFn2          :in       std_logic;                      --from FIFO#2
      RENn          :out      std_logic;                      --to FIFO
--INLINK----------------------------------
      AUX1IN        :in       std_logic;                      --from INLINK
      AUX2IN        :in       std_logic;                      --from INLINK
--OUTLINK---------------------------------
      RST           :out      std_logic;                      --to OUTLINK
      AUX1OUT       :out      std_logic;                      --to OUTLINK
      AUX2OUT       :out      std_logic;                      --to OUTLINK
      OUTPORT1      :out      std_logic;                      --to OUTLINK
      OUTPORT2      :out      std_logic;                      --to OUTLINK
      PAR           :out      std_logic_vector( 3 downto 0);  --to OUTLINK
--OTHER-----------------------------------
      LED           :out      std_logic_vector( 7 downto 0);  --to LED
      O_OPTIN       :in       std_logic_vector( 1 downto 0);  --to OUTLINK
      O_OPTOUT      :out      std_logic_vector( 1 downto 0);  --to OUTLINK
      I_OPTIN       :in       std_logic_vector( 1 downto 0);  --to INLINK
      I_OPTOUT      :out      std_logic_vector( 1 downto 0)   --to INLINK

      );
end cont2;


architecture MAINBODY of cont2 is

--signals for processing------------------
--signal INT_BTERMn   :std_logic;                        --
signal INT_ADR      :std_logic_vector(14 downto 0);    --
signal INT_DATA     :std_logic_vector(31 downto 0);    --
signal BUS_E        :std_logic;                        --
signal ADSnW1       :std_logic;                        --
signal ADSnW2       :std_logic;                        --
signal ADSnW3       :std_logic;                        --
signal INT_RAM      :std_logic_vector(31 downto 0);    --
signal INTWYn       :std_logic;                        --
signal DATA_SELECT  :std_logic_vector( 1 downto 0);    --select read data to 9080
signal INT_RSTn     :std_logic;                        --reset internal logic
signal INT_RAM_DATA :std_logic_vector(31 downto 0);    --store data from RAM
signal INT_READY    :std_logic;                        --for BIDIRECT signal
signal LEDCOUNT     :std_logic_vector(25 downto 0);    --for test LED
signal INTREN       :std_logic;                        --for test LED
signal ERR_ADR1     :std_logic_vector(14 downto 0);    --inlink par_err_adr spool
signal ERR_ADR2     :std_logic_vector(14 downto 0);    --inlink par_err_adr spool
signal ERR_ADR3     :std_logic_vector(14 downto 0);    --inlink par_err_adr spool
signal ERR_ADR4     :std_logic_vector(14 downto 0);    --inlink par_err_adr spool
--signal TEST_RAM0    :std_logic_vector(31 downto 0);    --store write adr
--signal TEST_RAM1    :std_logic_vector(31 downto 0);    --store write data
signal WADR         :std_logic_vector( 1 downto 0);    --select write data from 9080
signal INT_OUTPORT1 :std_logic;
signal INT_OUTPORT2 :std_logic;
signal INT_DATAREAD :std_logic;
signal INT_DRST     :std_logic;

--register for status information---------
signal INT_PAR_ERR  :std_logic_vector( 3 downto 0);    --
signal INT_ERR_ADR  :std_logic_vector(14 downto 0);    --
signal INT_AUX1IN   :std_logic;                        --inlink AUX1
signal INT_AUX2IN   :std_logic;                        --inlink AUX2
signal INT_AUX1OUT  :std_logic;                        --outlink AUX1
signal INT_AUX2OUT  :std_logic;                        --outlink AUX2
signal INT_COUNT    :std_logic_vector(14 downto 0);    --inlink write address
signal ERR_ADR      :std_logic_vector(14 downto 0);    --inlink par_err_adr
signal ERR_FLAG     :std_logic_vector( 3 downto 0);    --inlink par_err_flag
signal INT_STATEREAD:std_logic;

signal force_ren : std_logic;

begin

--##common part#################################################################

----control LED brightness----------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      LED(7) <= '1';
      LED(6) <= '1';
      LED(5) <= '1';
      LED(4) <= INT_RSTn;
      LED(3) <=	INT_OUTPORT1;
      LED(2) <= INT_OUTPORT2;
      LED(1) <= INT_DATAREAD;
      LED(0) <= INT_STATEREAD;
    end if;
  end process;

----make option signal--------------------
  O_OPTOUT <= "00";
  I_OPTOUT <= "11";

----make INT_RSTn from RST----------------
  process (RESETn) begin
      INT_RSTn <= RESETn;
  end process;

----make INT_AUXout-----------------------
  LRESETn <= '1';
--  INT_AUX1OUT <= '1';
--  INT_AUX2OUT <= '1';
--  INT_DRST    <= '1';

----make LHOLDA by LHOLD--------------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      LHOLDA <= LHOLD;
    end if;
  end process;

----make signal BUS_E-----------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      if (LHOLD = '0') then
        BUS_E <= '0';
      elsif (ADSn = '0') then
        BUS_E <= '1';
      elsif (BLASTn = '0') then
        BUS_E <= '0';
      end if;
    end if;
  end process;

----make signal DATA_SELECT to read data & write port---------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      if (ADSn = '0' and LWR = '0') then
        DATA_SELECT <= LAD(19 downto 18);
      end if;
    end if;
  end process;

----fetch AUX signal from inlink------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      INT_AUX1IN <= AUX1IN;
      INT_AUX2IN <= AUX2IN;
    end if;
  end process;
  
----make signal ADSnW2 (= 2 cycle delayed ADSn)--------------- 
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      ADSnW1 <= ADSn;
      ADSnW2 <= ADSnW1;                   --use in making READYin
      ADSnW3 <= ADSnW2;
    end if;
  end process;

----make signal READYin---------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      if (LHOLD = '0') then
        READYin <= '1';
        INT_READY <= '1';
      else
        if (LWR = '0') then              --read cycle (from INLINK)
          if (ADSnW3 = '0') then
            READYin <= '0';              --begin read transaction after 3 wait
            INT_READY <= '0';
          elsif (BLASTn = '0') then
            READYin <= '1';              --end read transaction
            INT_READY <= '1';
          end if;
        else                             --write cycle (to OUTLINK)
          if (ADSn = '0') then
            READYin <= '0';              --begin write transaction after 0 wait
          elsif (BLASTn = '0') then
            READYin <= '1';              --end write transaction
          end if;
        end if;
      end if;
    end if;
  end process;

----make INT_RAM_DATA-----------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      INT_RAM_DATA <= RAM_DATA;
    end if;
  end process;

--##read from inlink or internal to PCI###################################

----output read address to D.P.RAM----------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      if (LWR = '0') then
        if (ADSn = '0') then
          INT_ADR <= LAD(16 downto 2) + "000000000000001";
          RAM_ADR <= LAD(16 downto 2);
        else
          INT_ADR <= INT_ADR + "000000000000001";
          RAM_ADR <= INT_ADR;
        end if;
      end if;
    end if;
  end process;        

----select & output data to 9080------------------------------
  process (LWR,INT_READY,INT_RAM_DATA,data_select,
           int_aux1in,int_aux2in,int_aux1out,int_aux2out,
           int_drst, err_adr,int_count) begin
    if ((LWR = '0') and (INT_READY = '0')) then
      if (DATA_SELECT = "10") then
        LAD               <= INT_RAM_DATA;          --output data from inlink
        INT_DATAREAD <= '1';
        INT_STATEREAD<= '0';
      elsif (DATA_SELECT = "00") then
        LAD( 0)           <= INT_AUX1IN;
        LAD( 3 downto  1) <= "000";
        LAD( 4)           <= INT_AUX2IN;
        LAD( 7 downto  5) <= "000";
        LAD( 8)           <= INT_AUX1OUT;
        LAD(11 downto  9) <= "000";
        LAD(12)           <= INT_AUX2OUT;
        LAD(15 downto 13) <= "000";
        LAD(16)           <= INT_DRST;
        LAD(19 downto 17) <= "000";
        LAD(31 downto 20) <= "000000000000";
        INT_DATAREAD <= '0';
        INT_STATEREAD<= '1';
      elsif(DATA_SELECT = "11") then
        LAD(1 downto 0)   <= "00";
        LAD(16 downto 2)  <= ERR_ADR;               --output internal status
        LAD(27 downto 17) <= "00000000000";
        LAD(31 downto 28) <= ERR_FLAG;
        INT_DATAREAD <= '0';
        INT_STATEREAD<= '1';
      elsif(DATA_SELECT = "01") then
        LAD(1 downto 0)   <= "00";
        LAD(16 downto 2)  <= INT_COUNT;
        LAD(17) <= EFn1;
        LAD(18) <= EFn2;
        LAD(31 downto 19) <= "0000000000000";
        INT_DATAREAD <= '0';
        INT_STATEREAD<= '1';
      end if;
    else
      LAD <= "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
        INT_DATAREAD <= '0';
        INT_STATEREAD<= '0';
   end if;
  end process;

----input & output par from & to 9080---------------------------
--  process (LWR,INT_READY) begin
--    if ((LWR = '0') and (INT_READY = '0')) then
--      if ((DATA_SELECT = "10") or (DATA_SELECT = "11"))  then
--        LAD <= INT_RAM_DATA;                        --output data from inlink
--      elsif (DATA_SELECT = "00") then
--        LAD(14 downto 0) <= ERR_ADR;            --output internal status
--        LAD(15) <= ERR_FLAG;
--        LAD(16) <= INT_AUX1IN;
--        LAD(17) <= INT_AUX2IN;
--        LAD(31 downto 18) <= "00000000000000";
--      else
--        LAD <= "00000000000000000000000000000000";        
--      end if;
--    else
--      LAD <= "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
--    end if;
--  end process;



--##write from PCI to local bus#########################################
--make WADR from LAD------------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      if (ADSn = '0' and LWR = '1') then
        WADR <= LAD(18 downto 17);
      end if;
    end if;
  end process;

--select output HLINK port------------------------------------
  process (BCLK) begin
    if (RESETn = '0') then
      OUTPORT1 <= '0';
      OUTPORT2 <= '0';
      INT_OUTPORT1 <= '0';
      INT_OUTPORT2 <= '0';
   elsif (BCLK'event and BCLK = '1') then
      if ((BUS_E = '1') and (LWR = '1')) then
        if (WADR = "10") then              --output to BOTH on 40000 to 5ffff
          OUTPORT1 <= '1';              -- specialized for CB/PB version 2
          OUTPORT2 <= '1';
          INT_OUTPORT1 <= '1';
          INT_OUTPORT2 <= '1';
        elsif (WADR = "11") then           --output to HLINK2 on 60000 to 7ffff
          OUTPORT1 <= '0';
          OUTPORT2 <= '1';
          INT_OUTPORT1 <= '0';
          INT_OUTPORT2 <= '1';
        else                               --else don't output
          OUTPORT1 <= '0';
          OUTPORT2 <= '0';
          INT_OUTPORT1 <= '0';
          INT_OUTPORT2 <= '0';
        end if;
      else
        OUTPORT1 <= '0';
        OUTPORT2 <= '0';
        INT_OUTPORT1 <= '0';
        INT_OUTPORT2 <= '0';
       end if;
    end if;
  end process;

--output AUX--------------------------------------------------
  process (BCLK,INT_RSTn) begin
    if (INT_RSTn = '0') then
      AUX1OUT <= '0';
      AUX2OUT <= '0';
    elsif (BCLK'event and BCLK = '1') then
      AUX1OUT <= INT_AUX1OUT;
      AUX2OUT <= INT_AUX2OUT;
    end if;
  end process;

--output RST--------------------------------------------------
  process (BCLK,INT_RSTn) begin
    if (INT_RSTn = '0') then
        RST <= '0';
        BRSTn <= '0';
    elsif (BCLK'event and BCLK = '1') then
        RST <= INT_DRST;
        BRSTn <= '1';
    end if;
  end process;

----generate INT_AUXOUT & INT_RST --------------------------------------------
  process (BCLK,INT_RSTn) begin
    if (INT_RSTn = '0') then
      INT_AUX1OUT <= '1';
      INT_AUX2OUT <= '1';
      INT_DRST     <= '1';
    elsif (BCLK'event and BCLK = '1') then
      if ((LWR = '1') and (ADSnW1 = '0') and (WADR = "00")) then
        INT_AUX1OUT <= LAD(0);
        INT_AUX2OUT <= LAD(4);
        INT_DRST    <= LAD(8);
      end if;
    end if;
  end process;


--##error handling############################################################
--hold data from 9080 to make parity-------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      if (LWR ='1' and BUS_E = '1') then
        INT_DATA <= LAD;
      end if;
    end if;
  end process;

--make parity for outlink-------------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then

        PAR(0) <= ((INT_DATA( 0) xor INT_DATA( 1))
               xor (INT_DATA( 2) xor INT_DATA( 3))
               xor (INT_DATA( 4) xor INT_DATA( 5))
               xor (INT_DATA( 6) xor INT_DATA( 7)));

        PAR(1) <= ((INT_DATA( 8) xor INT_DATA( 9))
               xor (INT_DATA(10) xor INT_DATA(11))
               xor (INT_DATA(12) xor INT_DATA(13))
               xor (INT_DATA(14) xor INT_DATA(15)));

        PAR(2) <= ((INT_DATA(16) xor INT_DATA(17))
               xor (INT_DATA(18) xor INT_DATA(19))
               xor (INT_DATA(20) xor INT_DATA(21))
               xor (INT_DATA(22) xor INT_DATA(23)));

        PAR(3) <= ((INT_DATA(24) xor INT_DATA(25))
               xor (INT_DATA(26) xor INT_DATA(27))
               xor (INT_DATA(28) xor INT_DATA(29))
               xor (INT_DATA(30) xor INT_DATA(31)));
    end if;
  end process;

--check parity from inlink------------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then

        INT_PAR_ERR(0) <= ((RAM_DATA( 0) xor RAM_DATA( 1))
                       xor (RAM_DATA( 2) xor RAM_DATA( 3))
                       xor (RAM_DATA( 4) xor RAM_DATA( 5))
                       xor (RAM_DATA( 6) xor RAM_DATA( 7))
                       xor RAM_PAR(0));

        INT_PAR_ERR(1) <= ((RAM_DATA( 8) xor RAM_DATA( 9))
                       xor (RAM_DATA(10) xor RAM_DATA(11))
                       xor (RAM_DATA(12) xor RAM_DATA(13))
                       xor (RAM_DATA(14) xor RAM_DATA(15))
                       xor RAM_PAR(1));

        INT_PAR_ERR(2) <= ((RAM_DATA(16) xor RAM_DATA(17))
                       xor (RAM_DATA(18) xor RAM_DATA(19))
                       xor (RAM_DATA(20) xor RAM_DATA(21))
                       xor (RAM_DATA(22) xor RAM_DATA(23))
                       xor RAM_PAR(2));

        INT_PAR_ERR(3) <= ((RAM_DATA(24) xor RAM_DATA(25))
                       xor (RAM_DATA(26) xor RAM_DATA(27))
                       xor (RAM_DATA(28) xor RAM_DATA(29))
                       xor (RAM_DATA(30) xor RAM_DATA(31))
                       xor RAM_PAR(3));

    end if;
  end process;

--make address for ERR_ADR----------------------------------------
  process (BCLK) begin
    if (BCLK'event and BCLK = '1') then
      ERR_ADR1 <= INT_ADR;
      ERR_ADR2 <= ERR_ADR1;
      ERR_ADR3 <= ERR_ADR2;
      ERR_ADR4 <= ERR_ADR3;
    end if;
  end process;

--store error address to ERR_ADR AND flag & deflag ERR_FLAG---------------------
  process (BCLK)
  begin
      if (BCLK'event and BCLK = '1') then
          if ((LWR = '0') and (INT_READY = '0')) then
              if (DATA_SELECT = "00") then
                  ERR_FLAG <= "0000";
              elsif (DATA_SELECT = "10") then
                  if ((INT_PAR_ERR(0) = '1') or (INT_PAR_ERR(1) = '1')
                      or (INT_PAR_ERR(2) = '1') or (INT_PAR_ERR(3) = '1')) then
                      ERR_ADR <= ERR_ADR4;
                      ERR_FLAG <= INT_PAR_ERR;
                  end if;
              end if;
          end if;
      end if;
  end process;

--##count DPRAM write address###################################################
----if data is stored in FIFO, increase ADR COUNTER------------      
  process (BCLK,INT_RSTn) begin
    if (BCLK'event and BCLK = '1') then
      if (INT_RSTn = '0') then
        INT_COUNT <= "000000000000000";
      elsif ((LWR = '1') and (BUS_E = '1') and (WADR = "01")) then
        INT_COUNT <= LAD(16 downto 2);
      elsif ((EFn1 = '1') and (EFn2 = '1')) then
        INT_COUNT <= INT_COUNT + "000000000000001";
      end if;
    end if;
  end process;

  force_ren <=  '1' when (LWR = '1') and (BUS_E = '1') and (WADR = "01") else'0';
----if data is stored in FIFO, assert RENn to FIFO ------------------
        RENn <= not (( EFn1 and  EFn2) or  force_ren) ;
        INTREN <= (EFn1 and EFn2);

----if data is stored in FIFO, assert WYn to DPRAM-------------------      
  process (BCLK,INT_RSTn) begin
    if (INT_RSTn = '0') then                            --reset signal
      WYn <= '1';
    elsif (BCLK'event and BCLK = '1') then
      WYn <= not(EFn1 and EFn2);                        --FIFO is NOT empty
    end if;
  end process;

--output COUNT_OUT after 1 wait -------------------------------------
  process (BCLK,INT_RSTn) begin
    if (INT_RSTn = '0') then
      COUNT_OUT <= "000000000000000";
    elsif (BCLK'event and BCLK = '1') then
      COUNT_OUT <= INT_COUNT;
    end if;
  end process;


end MAINBODY;
